@@ -4,15 +4,16 @@ use 5.008008;
 use strict;
 use warnings;
 
-our $VERSION = '2.0';
+our $VERSION = '2.1';
 
 use Text::Sentence qw( split_sentences );
-use Lingua::NegEx;
+use Lingua::NegEx qw( negation_scope );
+
 
 use Class::MakeMethods (
   'Template::Hash:array' => [
         'target_phrases', 'skip_phrases',
-	'absolute_present_phrases', 'absolute_negative_phrases',
+		'absolute_present_phrases', 'absolute_negative_phrases',
   ],
   'Template::Hash:scalar' => [
         'orig_text', 'final_answer', 'ambiguous', 'start_phrase',
@@ -43,41 +44,43 @@ sub process_text {
 sub examine_text {
   my $self = shift;
   my $text = $self->orig_text;
+  return if ! $text;
   
-  if ( $self->start_phrase and $text =~ /$self->start_phrase(.*)\Z/ ) {
+  my $start_phrase = $self->start_phrase;
+  if ( $start_phrase and $text =~ /$start_phrase(.*)\Z/ix ) {
     $text = $1;
   }
-  
+  $text =~ s/\s+/ /gxms;
   # treat colon ':' like a period '.'
   $text =~ s/:/./g;
   
   my @sentences = split_sentences( $text );
   foreach my $line ( @sentences ) {
-    next if grep { $line =~ /\b$_\b/i } @{$self->skip_phrases};
+  
+    next if scalar grep { $line =~ /\b$_\b/i } @{$self->skip_phrases};
     next unless grep { $line =~ /\b$_\b/i } @{$self->target_phrases};
-	
+  	
     $self->target_sentence->{ $line } = 'present';
     my $n_scope = negation_scope( $line );
-    $self->negex_debug->{ $line } = $n_scope;
 
-    if ( $n_scope eq '-1' ) {
-      # affirmed
-      $self->target_sentence->{ $line } = 'present';
-    } elsif ( $n_scope eq '-2' ) {
-      # negated 
-      $self->target_sentence->{ $line } = 'absent';
-
-    } else {
-      # "Negated in this scope: $n_scope";
-      $n_scope =~ /(\d+)\s-\s(\d+)/;
-      my @words = split /\s/, $line;
-      my $term_in_scope;
-      foreach my $c ( $1 .. $2 ) {
-	    $term_in_scope = 1 if grep { $words[ $c ] =~ /$_/i } @{$self->target_phrases};
+     if ( $n_scope ) {
+      $self->negex_debug->{ $line } = @$n_scope[0] . ' - ' . @$n_scope[1];
+      my @words;
+      foreach ( split /\s/xms, $line ) {
+        s/\W//xms;
+        push @words, $_;
+      }
+      foreach my $c ( @$n_scope[0] .. @$n_scope[1] ) {
+        my @match = grep { $words[ $c ] =~ /$_/ixms } @{$self->target_phrases};
+		
+	    if ( scalar @match ) {
+          $self->target_sentence->{ $line } = 'absent';
+          last;
+        }
       }
-      $self->target_sentence->{ $line } = 'absent' if $term_in_scope;
     }
   }
+  
   if ( scalar keys %{$self->target_sentence} ) {
     my %final_answer;
     while ( my($sentence,$answer) = each %{$self->target_sentence} ) {
@@ -102,12 +105,13 @@ sub examine_text {
   } elsif ( ! scalar keys %{$self->target_sentence} ) {
     $self->final_answer( 'absent' );
   }
-  if ( grep { $text =~ /\w$_\w/i } @{$self->absolute_present_phrases} and $self->final_answer eq 'absent' ) {
+  
+  if ( grep { $text =~ /$_/i } @{$self->absolute_present_phrases} and $self->final_answer eq 'absent' ) {
     $self->final_answer( 'present' );
 	$self->ambiguous( 2 );
   }
-  if ( grep { $text =~ /\w$_\w/i } @{$self->absolute_negative_phrases} and $self->final_answer eq 'present' ) {
-     $self->final_answer( 'absent' );
+  if ( grep { $text =~ /$_/i } @{$self->absolute_negative_phrases} and $self->final_answer eq 'present' ) {
+    $self->final_answer( 'absent' );
 	$self->ambiguous( 3 ); 
   } 
 }
@@ -174,39 +178,55 @@ A tool to be used to look for the presence or absence of a clinical condition as
 
 The 'use case' for this is when performing a research project with a large number of records and you need to identify a subset based on a diagnostic entity, you can use this tool to reduce the number of charts that have to be manually examined. In this 'use case' I wanted to keep the sensitivity as high as possible in order to not miss real cases.
 
+The radiographic reports don't require textual preprocessing however clearly the selection of target_phrases and skip_phrases requires reading through reports to get a sense of what vocabulary is being used in the particular dataset that is being evaluated.
+
+Negated terms are identified using Lingua::NegEx which is a perl implementation of Wendy Chapman's NegEx algorithm.
   target_phrases( \@words );
 
-This is a list of words that describe the clinical entity in question. All forms of the entity in question need to explicitly stated since the package is currently not using lemmatization or stemming.
+This is a list of phrases that describe the clinical entity in question. All forms of the entity in question need to explicitly stated since the package is currently not using lemmatization or stemming.
 
   skip_phrases( \@skip );
 
-This is a list of words that can be used to eliminate sentences in the text that might confuse the extractor. For example most radiographic reports start with a brief description of the indication for the test. This statement may state the clinical entity in question but does not mean it is present in the study (ie. Indication: to rule out pulmonary embolism). 
+This is a list of phrases that can be used to eliminate sentences in the text that might confuse the extractor. For example most radiographic reports start with a brief description of the indication for the test. This statement may state the clinical entity in question but does not mean it is present in the study (ie. Indication: to rule out pulmonary embolism). 
 
 The radiographic reports don't require textual preprocessing however clearly the selection of target_phrases and skip_phrases requires reading through reports to get a sense of what vocabulary is being used in the particular dataset that is being evaluated.
 
+  absolute_negative_phrases( \@absolute_negative_assertions );
+ 
+This is a list of phrases which if present in the text mean the condition is certainly not there and all ambiguity checking can be skipped.
+ 
+  absolute_present_phrases( \@absolute_positive_assertions );
+
+This is a list of phrases which if present in the text mean the condition is certainly there and all ambiguity checking can be skipped.
+
+  start_phrase( $start_phrase );
+
+A phrase if present in the text which indicates where to focus the search.
+  
 Negated terms are identified using Lingua::NegEx which is a perl implementation of Wendy Chapman's NegEx algorithm.
 
 =head2 GETTING STARTED
 
 Define your extraction rules:
-	1. Choose your target phrases that describe the condition of interest.
-	2. Choose any skip phrases that will eliminate a sentence from an analysis if the phrase exists. For example, many radiology reports may have a sentence that states the indication for the study which might state the condition you are looking for but doesn't indicate that the condition was found in the study.
-	3. Choose any absolute present phrases to use which are phrases that if they are present in the text mean the condition is certainly there and all ambiguity checking can be skipped.
-	4. Choose any absolute negative phrases that mean the condition is absolutely not present and all ambiguity checking can be skipped.
-	5. Choose a start phrase which can be used to focus the search to only the text that occurs after this start phrase. For example, if there is an 'Impression:' section at the end of a report that summarizes findings then skip to that section to avoid possible ambiguity generated by the verbose description of findings in the full report.
+
+1. Choose your target phrases that describe the condition of interest.
+2. Choose any skip phrases that will eliminate a sentence from an analysis if the phrase exists. For example, many radiology reports may have a sentence that states the indication for the study which might state the condition you are looking for but doesn't indicate that the condition was found in the study.
+3. Choose any absolute present phrases to use which are phrases that if they are present in the text mean the condition is certainly there and all ambiguity checking can be skipped.
+4. Choose any absolute negative phrases that mean the condition is absolutely not present and all ambiguity checking can be skipped.
+5. Choose a start phrase which can be used to focus the search to only the text that occurs after this start phrase. For example, if there is an 'Impression:' section at the end of a report that summarizes findings then skip to that section to avoid possible ambiguity generated by the verbose description of findings in the full report.
 	
-	Only the target phrases are required.
+Only the target phrases are required.
 	
 Once defined the extractor object you created can be used to analyze target text. The analysis consists of:
 
-	1. If there is a start phrase defined, eliminate all text for analysis prior to the start phrase.
-	2. Change all colons ':' into periods '.' to treat them as sentence breaks
-	3. Split text into sentences using Text::Sentence
-	4. Examine each sentence for the presence of any skip phrases and if found, ignore the sentence.
-	5. Examine each sentence for the presence of any target phrases and if found evaluate for negation using Lingua:Negex.
-		-if no negation found, mark this sentence as 'present'
-		-if the target phrase is negated then mark the sentence as 'absent'
-	6. Go through all the flagged sentences and see if there is any discrepancy -- if so set the ambiguous flag. If there are more sentences that indicate absent than those that indicate present then mark the final answer as absent and vice versa. If there are an equal number of absent and present phrases mark the final answer as present.
+1. If there is a start phrase defined, eliminate all text for analysis prior to the start phrase.
+2. Change all colons ':' into periods '.' to treat them as sentence breaks
+3. Split text into sentences using Text::Sentence
+4. Examine each sentence for the presence of any skip phrases and if found, ignore the sentence.
+5. Examine each sentence for the presence of any target phrases and if found evaluate for negation using Lingua:Negex.
+ -if no negation found, mark this sentence as 'present'
+ -if the target phrase is negated then mark the sentence as 'absent'
+6. Go through all the flagged sentences and see if there is any discrepancy -- if so set the ambiguous flag. If there are more sentences that indicate absent than those that indicate present then mark the final answer as absent and vice versa. If there are an equal number of absent and present phrases mark the final answer as present.
 	
 =head2 EXPORT
 
